<?php
// $Id: feedapi_data.drush.inc,v 1.1.2.1 2009/08/28 14:25:40 alexb Exp $
/**
 * @file
 * Drush commands for FeedAPI Data.
 */

/**
 * Implementation of hook_drush_help().
 */
function feedapi_data_drush_help($section) {
  switch ($section) {
    case 'drush:feedapi_data dedupe':
      return dt('Delete duplicate items on a feed.');
  }
}

/**
 * Implementation of hook_drush_command().
 */
function feedapi_data_drush_command() {
  $items['feedapi_data dedupe'] = array(
    'callback' => 'feedapi_data_drush_dedupe',
    'description' => dt('Delete duplicate items on a feedapi_data feed.'),
    'arguments' => array(
       'nid' => 'The feed\'s node id. Mandatory.',
    ),
  );
  return $items;
}

/**
 * Command callback for deduping feedapi_data tables.
 */
function feedapi_data_drush_dedupe($nid) {
  // Input validation.
  if (empty($nid) || !is_numeric($nid)) {
    drush_print(dt('Usage: feedapi_data dedupe [feed node nid]'));
    return;
  }
  if (!$feed_node = node_load($nid)) {
    drush_print(dt('Node does not exist.'));
    return;
  }
  if (!in_array('feedapi_data', $feed_node->feed->processors)) {
    drush_print(dt('Node does not have a feedapi_data processor configured.'));
    return;
  }

  // Get unique fields.
  $unique_mappings = feedapi_mapper_get_uniques($feed_node);
  foreach ($unique_mappings as $mapping) {
    $unique_fields[$mapping['target'][1]] = db_escape_string($mapping['target'][1]);
  }

  // Query table for all feed items of this feed.
  $table = _feedapi_data_get($feed_node->type);
  $result = db_query('SELECT id, '. implode(', ', $unique_fields) .' FROM {'. db_escape_table($table->get('name')) .'} WHERE feed_nid = %d', $feed_node->nid);
  $uniques = $delete = array();
  $count = 0;

  // Go through every feed item and see whether it already exists.
  while ($item = db_fetch_object($result)) {
    $count++;
    foreach ($unique_fields as $unique) {
      if (!empty($item->$unique)) {
        if (!isset($uniques[$unique][$item->$unique])) {
          $uniques[$unique][$item->$unique] = $item->$unique;
        }
        else {
          // This item has been collected before on this unique flag, collect for deletion.
          $delete[$item->id] = $item->id;
        }
      }
    }
  }
  _feedapi_data_bulk_delete($feed_node, 'feed_item_table.id IN (%s)', implode(', ', $delete));
  drush_print(dt('Deleted !deleted out of !total items.', array('!deleted' => count($delete), '!total' => $count)));
}